@@ -1,10 +1,13 @@
 import re
+import urllib
 from xml.etree import ElementTree
 
 from . import __version__
 from .ns import _ns
 from .httpsession import HTTPSession
 from .models import Spreadsheet
+from .urls import construct_url
+from .utils import finditem
 from .exceptions import (AuthenticationError, SpreadsheetNotFound,
                          NoValidUrlKeyFound)
 
@@ -13,8 +16,6 @@ SPREADSHEETS_SERVER = 'spreadsheets.google.com'
 
 _url_key_re = re.compile(r'key=([^&#]+)')
 
-def finditem(func, seq):
-    return next((item for item in seq if func(item)))
 
 class Client(object):
     """A client class for communicating with Google's Date API.
@@ -123,30 +124,39 @@ class Client(object):
         return result
 
     def get_spreadsheets_feed(self, visibility='private', projection='full'):
-        uri = ('https://%s/feeds/spreadsheets/%s/%s'
-            % (SPREADSHEETS_SERVER, visibility, projection))
+        url = construct_url('spreadsheets',
+                            visibility=visibility, projection=projection)
 
-        r = self.session.get(uri)
+        r = self.session.get(url)
         return ElementTree.fromstring(r.read())
 
-    def get_worksheets_feed(self, spreadsheet_id,
+    def get_worksheets_feed(self, spreadsheet,
                             visibility='private', projection='full'):
-        uri = ('https://%s/feeds/worksheets/%s/%s/%s'
-            % (SPREADSHEETS_SERVER, spreadsheet_id, visibility, projection))
+        url = construct_url('worksheets', spreadsheet,
+                            visibility=visibility, projection=projection)
 
-        r = self.session.get(uri)
+        r = self.session.get(url)
         return ElementTree.fromstring(r.read())
 
-    def get_cells_feed(self, spreadsheet_id, worksheet_id, cell=None,
+    def get_cells_feed(self, worksheet,
+                       visibility='private', projection='full', params=None):
+
+        url = construct_url('cells', worksheet,
+                            visibility=visibility, projection=projection)
+
+        if params:
+            params = urllib.urlencode(params)
+            url = '%s?%s' % (url, params)
+
+        r = self.session.get(url)
+        return ElementTree.fromstring(r.read())
+
+    def get_cells_cell_id_feed(self, worksheet, cell_id,
                        visibility='private', projection='full'):
-        uri = ('https://%s/feeds/cells/%s/%s/%s/%s'
-            % (SPREADSHEETS_SERVER, spreadsheet_id, worksheet_id,
-               visibility, projection))
+        url = construct_url('cells_cell_id', worksheet, cell_id=cell_id,
+                            visibility=visibility, projection=projection)
 
-        if cell != None:
-            uri = '%s/%s' % (uri, cell)
-
-        r = self.session.get(uri)
+        r = self.session.get(url)
         return ElementTree.fromstring(r.read())
 
     def put_cell(self, url, data):
@@ -155,3 +165,11 @@ class Client(object):
         r = self.session.put(url, data, headers=headers)
 
         return ElementTree.fromstring(r.read())
+
+    def post_cells(self, worksheet, data):
+        headers = {'Content-Type': 'application/atom+xml'}
+        data = "<?xml version='1.0' encoding='UTF-8'?>%s" % data
+        url = construct_url('cells_batch', worksheet)
+        r = self.session.post(url, data, headers=headers)
+
+        return ElementTree.fromstring(r.read())
