@@ -1,6 +1,10 @@
 from xml.etree import ElementTree
+from xml.etree.ElementTree import Element, SubElement
 
 from .ns import _ns, _ns1
+from .urls import construct_url
+from .utils import finditem
+
 
 class Spreadsheet(object):
     """A model for representing a spreadsheet object.
@@ -12,11 +16,14 @@ class Spreadsheet(object):
         self.id = id_parts[-1]
         self._sheet_list = []
 
+    def get_id_fields(self):
+        return {'spreadsheet_id': self.id}
+
     def sheet_by_name(self, sheet_name):
         pass
 
     def _fetch_sheets(self):
-        feed = self.client.get_worksheets_feed(self.id)
+        feed = self.client.get_worksheets_feed(self)
         for elem in feed.findall(_ns('entry')):
             self._sheet_list.append(Worksheet(self, elem))
 
@@ -46,16 +53,18 @@ class Worksheet(object):
         self.client = spreadsheet.client
         self.id = feed_entry.find(_ns('id')).text.split('/')[-1]
 
+    def get_id_fields(self):
+        return {'spreadsheet_id': self.spreadsheet.id,
+                'worksheet_id': self.id}
+
     def _cell_addr(self, row, col):
         return 'R%sC%s' % (row, col)
 
     def _fetch_cells(self):
-        feed = self.client.get_cells_feed(self.spreadsheet.id, self.id)
+        feed = self.client.get_cells_feed(self)
         cells_list = []
         for elem in feed.findall(_ns('entry')):
-            c_elem = elem.find(_ns1('cell'))
-            cells_list.append(Cell(self, c_elem.get('row'),
-                                   c_elem.get('col'), c_elem.text))
+            cells_list.append(Cell(self, elem))
 
         return cells_list
 
@@ -65,11 +74,14 @@ class Worksheet(object):
         Fetch a cell in row `row` and column `col`.
 
         """
-        feed = self.client.get_cells_feed(self.spreadsheet.id,
-                                          self.id, self._cell_addr(row, col))
-        cell_elem = feed.find(_ns1('cell'))
-        return Cell(self, cell_elem.get('row'), cell_elem.get('col'),
-                    cell_elem.text)
+        feed = self.client.get_cells_cell_id_feed(self,
+                                                  self._cell_addr(row, col))
+        return Cell(self, feed)
+
+    def range(self, alphanum):
+        feed = self.client.get_cells_feed(self, params={'range': alphanum,
+                                                        'return-empty': 'true'})
+        return [Cell(self, elem) for elem in feed.findall(_ns('entry'))]
 
     def get_all_rows(self):
         """Return a list of lists containing worksheet's rows."""
@@ -136,22 +148,64 @@ class Worksheet(object):
 
     def update_cell(self, row, col, val):
         """Set new value to a cell."""
-        feed = self.client.get_cells_feed(self.spreadsheet.id,
-                                          self.id, self._cell_addr(row, col))
+        feed = self.client.get_cells_cell_id_feed(self,
+                                                  self._cell_addr(row, col))
         cell_elem = feed.find(_ns1('cell'))
         cell_elem.set('inputValue', val)
-        edit_link = filter(lambda x: x.get('rel') == 'edit',
-                feed.findall(_ns('link')))[0]
+        edit_link = finditem(lambda x: x.get('rel') == 'edit',
+                feed.findall(_ns('link')))
         uri = edit_link.get('href')
 
         self.client.put_cell(uri, ElementTree.tostring(feed))
 
+    def _create_update_feed(self, cell_list):
+        feed = Element('feed',
+                      {'xmlns': 'http://www.w3.org/2005/Atom',
+                       'xmlns:batch': 'http://schemas.google.com/gdata/batch',
+                       'xmlns:gs': 'http://schemas.google.com/spreadsheets/2006'})
+
+        id_elem = SubElement(feed, 'id')
+
+        id_elem.text = construct_url('cells', self)
+
+        for cell in cell_list:
+            entry = SubElement(feed, 'entry')
+
+            SubElement(entry, 'batch:id').text = cell.element.find(_ns('title')).text
+            SubElement(entry, 'batch:operation', {'type': 'update'})
+            SubElement(entry, 'id').text = cell.element.find(_ns('id')).text
+
+            edit_link = finditem(lambda x: x.get('rel') == 'edit',
+                    cell.element.findall(_ns('link')))
+
+            SubElement(entry, 'link', {'rel': 'edit',
+                                       'type': edit_link.get('type'),
+                                       'href': edit_link.get('href')})
+
+            SubElement(entry, 'gs:cell', {'row': cell.row,
+                                          'col': cell.col,
+                                          'inputValue': cell.value})
+        return feed
+
+    def update_cells(self, cell_list):
+        """Cells batch update."""
+        feed = self._create_update_feed(cell_list)
+        self.client.post_cells(self, ElementTree.tostring(feed))
+
 
 class Cell(object):
     """A model for cell object.
 
     """
-    def __init__(self, worksheet, row, col, value):
-        self.row = row
-        self.col = col
-        self.value = value
+    def __init__(self, worksheet, element):
+        self.element = element
+        cell_elem = element.find(_ns1('cell'))
+        self.row = cell_elem.get('row')
+        self.col = cell_elem.get('col')
+        self.value = cell_elem.text
+
+    def __repr__(self):
+        return '<%s R%sC%s "%s">' % (self.__class__.__name__,
+                                     self.row,
+                                     self.col,
+                                     self.value)
